"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AxiosDebugHelper = void 0;
/**
 * Implements AxiosDebug for classes
 */
class AxiosDebugHelper {
    /**
     *
     * @param instance AxiosInstance
     * @param logger Logger
     */
    constructor(instance, logger) {
        this.instance = instance;
        this.logger = logger;
        this.isAbsoluteURL = require('axios/lib/helpers/isAbsoluteURL');
        this.buildURL = require('axios/lib/helpers/buildURL');
        this.combineURLs = require('axios/lib/helpers/combineURLs');
        this.requestUrl = '';
        this.addLogger(instance);
    }
    getURL(config) {
        let url = config.url;
        if (config.baseURL && !this.isAbsoluteURL(url)) {
            url = this.combineURLs(config.baseURL, url);
        }
        return this.buildURL(url, config.params, config.paramsSerializer);
    }
    addLogger(instance) {
        instance.interceptors.request.use((config) => {
            this.request(config);
            return config;
        });
        instance.interceptors.response.use((response) => {
            this.response(response);
            return response;
        }, (error) => {
            this.error(error);
            throw error;
        });
    }
    request(config) {
        var _a;
        this.requestUrl = this.getURL(config);
        Object.defineProperty(config, this.requestUrl, { value: this.requestUrl });
        this.logger.debug(((_a = config.method) === null || _a === void 0 ? void 0 : _a.toUpperCase()) + ' ' + this.requestUrl);
    }
    response(response) {
        var _a, _b;
        this.logger.debug(response.status +
            ' ' +
            response.statusText +
            ' (' +
            ((_b = (_a = response.config) === null || _a === void 0 ? void 0 : _a.method) === null || _b === void 0 ? void 0 : _b.toUpperCase()) +
            ' ' +
            this.requestUrl +
            ')');
    }
    error(error) {
        var _a, _b;
        if (error.config) {
            this.logger.error(error.name + ': ' + error.message, '(' + ((_b = (_a = error.config) === null || _a === void 0 ? void 0 : _a.method) === null || _b === void 0 ? void 0 : _b.toUpperCase()) + ' ' + this.requestUrl + ')');
        }
        else {
            this.logger.error(error.name + ': ' + error.message);
        }
    }
}
exports.AxiosDebugHelper = AxiosDebugHelper;
//# sourceMappingURL=axiosDebugHelper.js.map