"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DingzLogger = void 0;
const chalk_1 = __importDefault(require("chalk"));
/**
 * Drop-in helper class used to prefix debug messages with a colored dingz name
 */
class DingzLogger {
    /**
     *
     * @param logger the homebridge logger to do the actual logging
     */
    constructor(dingzPrefix, logger) {
        this.dingzPrefix = dingzPrefix;
        this.logger = logger;
    }
    /**
     * Logger.log like method
     * @param logLevel Log level (Logger.LogLevel)
     * @param message The message
     * @param parameters Additional parameter
     */
    log(logLevel, message, ...parameters) {
        message = chalk_1.default.magentaBright(`[${this.dingzPrefix}] `) + message;
        this.logger.log(logLevel, message, ...parameters);
    }
    info(message, ...parameters) {
        this.log("info" /* LogLevel.INFO */, `${message}`, ...parameters);
    }
    warn(message, ...parameters) {
        this.log("warn" /* LogLevel.WARN */, `${message}`, ...parameters);
    }
    error(message, ...parameters) {
        this.log("error" /* LogLevel.ERROR */, `${message}`, ...parameters);
    }
    debug(message, ...parameters) {
        this.log("debug" /* LogLevel.DEBUG */, `${message}`, ...parameters);
    }
}
exports.DingzLogger = DingzLogger;
//# sourceMappingURL=dingzLogHelper.js.map