"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MyStromLightbulbAccessory = void 0;
const simple_color_converter_1 = __importDefault(require("simple-color-converter"));
const qs_1 = __importDefault(require("qs"));
const dingzDaBaseAccessory_1 = require("./lib/dingzDaBaseAccessory");
/**
 * Platform Accessory
 * An instance of this class is created for each accessory your platform registers
 * Each accessory may expose multiple services of different service types.
 */
class MyStromLightbulbAccessory extends dingzDaBaseAccessory_1.DingzDaBaseAccessory {
    constructor(_platform, _accessory) {
        var _a, _b, _c;
        super(_platform, _accessory);
        this._platform = _platform;
        this._accessory = _accessory;
        this.lightbulbState = {
            on: false,
            color: '0;0;0',
            hue: 0,
            saturation: 0,
            value: 0,
            mode: 'hsv',
            power: 0,
        };
        // Set Base URL
        this.mystromDeviceInfo = this.device.hwInfo;
        this.platform.log.debug('Setting informationService Characteristics ->', this.device.model);
        this.accessory
            .getService(this.platform.Service.AccessoryInformation)
            .setCharacteristic(this.platform.Characteristic.Manufacturer, 'MyStrom AG')
            .setCharacteristic(this.platform.Characteristic.AppMatchingIdentifier, 'ch.mystrom.iOSApp')
            .setCharacteristic(this.platform.Characteristic.Model, this.device.model)
            .setCharacteristic(this.platform.Characteristic.FirmwareRevision, (_a = this.mystromDeviceInfo.version) !== null && _a !== void 0 ? _a : 'N/A')
            .setCharacteristic(this.platform.Characteristic.SerialNumber, this.device.mac);
        this.lightbulbService =
            (_b = this.accessory.getService(this.platform.Service.Lightbulb)) !== null && _b !== void 0 ? _b : this.accessory.addService(this.platform.Service.Lightbulb);
        this.lightbulbService.setCharacteristic(this.platform.Characteristic.Name, (_c = this.device.name) !== null && _c !== void 0 ? _c : `${this.accessory.context.device.model} Bulb`);
        // register handlers for the On/Off Characteristic
        this.lightbulbService
            .getCharacteristic(this.platform.Characteristic.On)
            .on("set" /* CharacteristicEventTypes.SET */, this.setOn.bind(this)) // SET - bind to the `setOn` method below
            .on("get" /* CharacteristicEventTypes.GET */, this.getOn.bind(this)); // GET - bind to the `getOn` method below
        // register handlers for the Brightness Characteristic
        this.lightbulbService
            .getCharacteristic(this.platform.Characteristic.Brightness)
            .on("set" /* CharacteristicEventTypes.SET */, this.setBrightness.bind(this)) // SET - bind to the 'setBrightness` method below
            .on("get" /* CharacteristicEventTypes.GET */, this.getBrightness.bind(this)); // SET - bind to the 'setBrightness` method below
        // register handlers for the Brightness Characteristic
        this.lightbulbService
            .getCharacteristic(this.platform.Characteristic.Hue)
            .on("set" /* CharacteristicEventTypes.SET */, this.setHue.bind(this)) // SET - bind to the 'setBrightness` method below
            .on("get" /* CharacteristicEventTypes.GET */, this.getHue.bind(this)); // SET - bind to the 'setBrightness` method below
        // register handlers for the Brightness Characteristic
        this.lightbulbService
            .getCharacteristic(this.platform.Characteristic.Saturation)
            .on("set" /* CharacteristicEventTypes.SET */, this.setSaturation.bind(this)) // SET - bind to the 'setBrightness` method below
            .on("get" /* CharacteristicEventTypes.GET */, this.getSaturation.bind(this)); // SET - bind to the 'setBrightness` method below
    }
    // Get updated device info and update the corresponding values
    getDeviceStateUpdate() {
        return this.getDeviceReport(this.device.mac)
            .then((report) => {
            // FIXME: Add 'mono' as well
            if (report && report.mode === 'hsv') {
                // push the new value to HomeKit
                this.lightbulbState = report;
                const hsv = report.color.split(';');
                this.lightbulbState.hue = parseInt(hsv[0]);
                this.lightbulbState.saturation = parseInt(hsv[1]);
                this.lightbulbState.value = parseInt(hsv[2]);
            }
            else if (report && report.mode === 'rgb') {
                // rgbw
                const hsv = new simple_color_converter_1.default({
                    color: `hex #${report.color}`,
                    to: 'hsv',
                });
                this.lightbulbState.hue = hsv.c;
                this.lightbulbState.saturation = hsv.s;
                this.lightbulbState.value = hsv.i;
            }
            this.lightbulbService
                .getCharacteristic(this.platform.Characteristic.Hue)
                .updateValue(this.lightbulbState.hue);
            this.lightbulbService
                .getCharacteristic(this.platform.Characteristic.Saturation)
                .updateValue(this.lightbulbState.saturation);
            this.lightbulbService
                .getCharacteristic(this.platform.Characteristic.Brightness)
                .updateValue(this.lightbulbState.value);
            this.lightbulbService
                .getCharacteristic(this.platform.Characteristic.On)
                .updateValue(this.lightbulbState.on);
            // Return promise
            return Promise.resolve();
        })
            .catch((e) => {
            return Promise.reject(e);
        });
    }
    identify() {
        this.platform.log.debug('Identify! -> Who am I? I am', this.accessory.displayName);
    }
    setOn(value, callback) {
        this.lightbulbState.on = value;
        this.setDeviceLightbulb(callback);
    }
    getOn(callback) {
        const isOn = this.lightbulbState.on;
        this.platform.log.debug('Get Characteristic On ->', isOn);
        callback(this.reachabilityState, isOn);
    }
    async setHue(value, callback) {
        this.lightbulbState.hue = value;
        this.setDeviceLightbulb(callback);
    }
    getHue(callback) {
        const hue = this.lightbulbState.hue;
        callback(this.reachabilityState, hue);
    }
    async setSaturation(value, callback) {
        this.lightbulbState.saturation = value;
        this.setDeviceLightbulb(callback);
    }
    getSaturation(callback) {
        const saturation = this.lightbulbState.saturation;
        callback(this.reachabilityState, saturation);
    }
    async setBrightness(value, callback) {
        this.lightbulbState.value = value;
        this.setDeviceLightbulb(callback);
    }
    getBrightness(callback) {
        const brightness = this.lightbulbState.value;
        callback(this.reachabilityState, brightness);
    }
    // Set individual dimmer
    setDeviceLightbulb(callback) {
        const state = this.lightbulbState;
        const color = `${state.hue};${state.saturation};${state.value}`;
        const setDimmerUrl = `${this.baseUrl}/api/v1/device/${this.device.mac}`;
        const data = qs_1.default.stringify({
            action: state.on ? 'on' : 'off',
            color: color !== null && color !== void 0 ? color : undefined,
            mode: color ? 'hsv' : undefined,
        }, { encode: false });
        return this.request
            .post(setDimmerUrl, data)
            .catch(this.handleRequestErrors.bind(this))
            .finally(() => {
            callback(this.reachabilityState);
        });
    }
    async getDeviceReport(mac) {
        const reportUrl = `${this.baseUrl}/api/v1/device/`;
        return await this.request(reportUrl).then((response) => {
            return response.data[mac];
        });
    }
}
exports.MyStromLightbulbAccessory = MyStromLightbulbAccessory;
//# sourceMappingURL=myStromLightbulbAccessory.js.map