"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DingzDaHomebridgePlatform = void 0;
const cockatiel_1 = require("cockatiel");
const dgram_1 = require("dgram");
const axios_1 = __importDefault(require("axios"));
const axios_retry_1 = __importDefault(require("axios-retry"));
const bodyParser = __importStar(require("body-parser"));
const intervals_for_humans_1 = __importDefault(require("intervals-for-humans"));
const chalk_1 = __importDefault(require("chalk"));
const is_valid_host_1 = __importDefault(require("is-valid-host"));
const os = __importStar(require("os"));
const e = require("express");
const myStromTypes_1 = require("./lib/myStromTypes");
const commonTypes_1 = require("./lib/commonTypes");
const errors_1 = require("./lib/errors");
const settings_1 = require("./settings");
const platformEventBus_1 = require("./lib/platformEventBus");
// Accessory classes
const dingzAccessory_1 = require("./dingzAccessory");
const myStromSwitchAccessory_1 = require("./myStromSwitchAccessory");
const myStromLightbulbAccessory_1 = require("./myStromLightbulbAccessory");
const myStromButtonAccessory_1 = require("./myStromButtonAccessory");
const myStromPIRAccessory_1 = require("./myStromPIRAccessory");
const myStromButtonPlusAccessory_1 = require("./myStromButtonPlusAccessory");
// Define a policy that will retry 20 times at most
const retry = cockatiel_1.Policy.handleAll()
    .retry()
    .attempts(20)
    .exponential({
    maxDelay: (0, intervals_for_humans_1.default)('1h'),
    initialDelay: (0, intervals_for_humans_1.default)('2s'),
});
// Create a circuit breaker that'll stop calling the executed function for 10
// seconds if it fails 5 times in a row. This can give time for e.g. a database
// to recover without getting tons of traffic.
const circuitBreaker = cockatiel_1.Policy.handleAll().circuitBreaker(10 * 1000, new cockatiel_1.ConsecutiveBreaker(5));
const retryWithBreaker = cockatiel_1.Policy.wrap(retry, circuitBreaker);
/**
 * HomebridgePlatform
 * This class is the main constructor for the plugin, this is where you should
 * parse the user config and discover/register accessories with Homebridge.
 */
class DingzDaHomebridgePlatform {
    constructor(log, config, api) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.Service = this.api.hap.Service;
        this.Characteristic = this.api.hap.Characteristic;
        this.eb = new platformEventBus_1.PlatformEventBus();
        // this is used to track restored cached accessories
        this.accessories = {};
        this.discovered = new Map();
        this.ignored = new Map();
        this.app = e();
        this.handleError = (e) => {
            var _a, _b, _c;
            if (e.response) {
                this.log.error(`HTTP ${e.code} ${e.message} ${(_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.statusText) !== null && _b !== void 0 ? _b : ' '}--> ${(_c = e.response.config.url) !== null && _c !== void 0 ? _c : 'NO URL'}`);
            }
            else if (e.request) {
                this.log.error(`HTTP ${e.code} ${e.message}}--> ` + e.request.url);
            }
            else {
                this.log.error('HTTP Response Error ->' + e.config.url);
                this.log.error(e.message);
            }
        };
        (0, axios_retry_1.default)(axios_1.default, { retries: 5, retryDelay: axios_retry_1.default.exponentialDelay });
        // Adds ignored devices from Config
        if (this.config.ignore) {
            for (const device of this.config.ignore) {
                // Only add to map if mac is set
                if (device.mac) {
                    const mac = device.mac.toUpperCase();
                    this.ignored.set(mac, {
                        comment: (device === null || device === void 0 ? void 0 : device.comment) || '',
                        isignored: false,
                    });
                    this.log.info(chalk_1.default.redBright('[Platform]'), `Will be ignoring device ${(device === null || device === void 0 ? void 0 : device.comment) || ''} with MAC Address ${mac}`);
                }
            }
        }
        // When this event is fired it means Homebridge has restored all cached accessories from disk.
        // Dynamic Platform plugins should only register new accessories after this event was fired,
        // in order to ensure they weren't added to homebridge already. This event can also be used
        // to start discovery of new accessories.
        this.api.on("didFinishLaunching" /* APIEvent.DID_FINISH_LAUNCHING */, () => {
            this.log.debug(chalk_1.default.redBright('[Platform]'), 'Finished initializing platform:', this.config.name);
            // Adds decvices from Config
            if (this.config.devices) {
                this.addDevices();
            }
            // Discovers devices from UDP
            this.setupDeviceDiscovery();
            // set-up the callback server ...
            this.callbackServer();
            this.eb.on("requestStateUpdate" /* PlatformEvent.REQUEST_STATE_UPDATE */, () => {
                this.log.debug('Event -> PlatformEvent.REQUEST_STATE_UPDATE');
            });
            // Set the update interval in seconds
            const updateInterval = !this.config.pollerInterval ||
                this.config.pollerInterval < settings_1.STATE_UPDATE_INTERVAL
                ? settings_1.STATE_UPDATE_INTERVAL
                : this.config.pollerInterval;
            this.log.debug(`Platform poll interval: ${updateInterval}s`);
            setInterval(() => {
                this.eb.emit("requestStateUpdate" /* PlatformEvent.REQUEST_STATE_UPDATE */);
            }, updateInterval * 1000);
        });
    }
    /**
     * This function is invoked when homebridge restores cached accessories from disk at startup.
     * It should be used to setup event handlers for characteristics and update respective values.
     */
    configureAccessory(accessory) {
        this.log.info(chalk_1.default.redBright('[Platform]'), 'Restoring accessory from cache:', accessory.displayName, '->', accessory.context.device.accessoryClass);
        const context = accessory.context;
        if (context.device && context.device.accessoryClass) {
            if (this.ignored.has(context.device.mac.toUpperCase())) {
                this.log.warn(chalk_1.default.redBright('[Platform]'), 'This cached accessory is also in the list of ignored devices.', 'The plugin will continue loading it.', 'Consider manually removing it from Homebridge.');
            }
            this.log.debug('Restoring accessory of class ->', context.device.accessoryClass);
            switch (context.device.accessoryClass) {
                case 'DingzDaAccessory':
                case 'DingzAccessory':
                    // Add the restored accessory
                    this.accessories[accessory.UUID] = new dingzAccessory_1.DingzAccessory(this, accessory);
                    break;
                case 'MyStromSwitchAccessory':
                    this.accessories[accessory.UUID] = new myStromSwitchAccessory_1.MyStromSwitchAccessory(this, accessory);
                    break;
                case 'MyStromLightbulbAccessory':
                    this.accessories[accessory.UUID] = new myStromLightbulbAccessory_1.MyStromLightbulbAccessory(this, accessory);
                    break;
                case 'MyStromButtonAccessory':
                    this.accessories[accessory.UUID] = new myStromButtonAccessory_1.MyStromButtonAccessory(this, accessory);
                    break;
                case 'MyStromButtonPlusAccessory':
                    this.accessories[accessory.UUID] = new myStromButtonPlusAccessory_1.MyStromButtonPlusAccessory(this, accessory);
                    break;
                case 'MyStromPIRAccessory':
                    // add the restored accessory to the accessories cache so we can track if it has already been registered
                    this.accessories[accessory.UUID] = new myStromPIRAccessory_1.MyStromPIRAccessory(this, accessory);
                    break;
                default:
                    this.log.warn('No Accessory type defined for Accessory', accessory.displayName, 'can not restore');
                    return;
            }
        }
        else {
            this.log.warn('No Accessory device context for Accessory', accessory.displayName, 'can not restore');
        }
    }
    addDevices() {
        // loop over the discovered devices and register each one if it has not already been registered
        for (const device of this.config.devices) {
            // Call addDevice, retry until found
            this.log.info('Add Device from config: ', device.type, '->', device.name);
            switch (device.type) {
                case 'dingz':
                    retryWithBreaker.execute(() => {
                        var _a;
                        return this.addDingzDevice({
                            address: device.address,
                            name: device.name,
                            token: (_a = device.token) !== null && _a !== void 0 ? _a : this.config.globalToken,
                        });
                    });
                    break;
                case 'myStromSwitch':
                    retryWithBreaker.execute(() => {
                        var _a;
                        return this.addMyStromSwitchDevice({
                            address: device.address,
                            name: device.name,
                            token: (_a = device.token) !== null && _a !== void 0 ? _a : this.config.globalToken,
                        });
                    });
                    break;
                case 'myStromBulb':
                case 'myStromLED': // Share the same code
                    retryWithBreaker.execute(() => {
                        var _a;
                        return this.addMyStromLightbulbDevice({
                            address: device.address,
                            name: device.name,
                            token: (_a = device.token) !== null && _a !== void 0 ? _a : this.config.globalToken,
                        });
                    });
                    break;
                case 'myStromPIR':
                    retryWithBreaker.execute(() => {
                        var _a;
                        return this.addMyStromPIRDevice({
                            address: device.address,
                            name: device.name,
                            token: (_a = device.token) !== null && _a !== void 0 ? _a : this.config.globalToken,
                        });
                    });
                    break;
                case 'myStromButtonPlus':
                    this.log.info('Device type', device.deviceType, 'is currently unsupported for manual addition. Will skip.');
                    break;
                default:
                    this.log.info('Device type', device.deviceType, 'is currently unsupported. Will skip.');
                    break;
            }
        }
    }
    // Add one device based on address and name
    async addDingzDevice({ address, name, token, existingAccessory, }) {
        // Run a discovery of changed things every 10 seconds
        this.log.debug(`addDingzDevice() --> Add configured device -> ${name} (${address})`);
        return await dingzAccessory_1.DingzAccessory.getConfig({
            address: address,
            token: token,
        })
            .then(({ dingzDevices, dingzConfig }) => {
            var _a;
            this.log.debug('Got Device ->', JSON.stringify(dingzDevices));
            if (typeof dingzDevices !== 'undefined') {
                const keys = Object.keys(dingzDevices);
                const mac = keys[0]; // keys[0]
                const info = dingzDevices[mac];
                if (info.type !== 'dingz') {
                    throw new errors_1.InvalidTypeError(`Device ${name} at ${address} is of the wrong type (${info.type} instead of "dingz")`);
                }
                const deviceInfo = {
                    name: dingzConfig.systemConfig.dingz_name &&
                        dingzConfig.systemConfig.dingz_name !== ''
                        ? dingzConfig.systemConfig.dingz_name
                        : name,
                    address: address,
                    mac: mac.toUpperCase(),
                    token: token,
                    model: (_a = info.front_hw_model) !== null && _a !== void 0 ? _a : 'dingz',
                    hwInfo: info,
                    accessoryClass: 'DingzAccessory',
                };
                const uuid = this.api.hap.uuid.generate(deviceInfo.mac);
                // check that the device has not already been registered by checking the
                // cached devices we stored in the `configureAccessory` method above
                if (!this.accessories[uuid]) {
                    this.log.info(`Registering ${existingAccessory ? 'existing' : 'new'} accessory: ${deviceInfo.name}`);
                    // create a new accessory
                    const accessory = existingAccessory !== null && existingAccessory !== void 0 ? existingAccessory : new this.api.platformAccessory(deviceInfo.name, uuid);
                    // store a copy of the device object in the `accessory.context`
                    // the `context` property can be used to store any data about the accessory you may need
                    accessory.context.device = deviceInfo;
                    accessory.context.config = dingzConfig;
                    // link the accessory to your platform
                    try {
                        if (!existingAccessory) {
                            this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                        }
                        // create the accessory handler (which will add services as needed)
                        // this is imported from `dingzDaAccessory.ts`
                        // push into accessory cache
                        const dingzDaAccessory = new dingzAccessory_1.DingzAccessory(this, accessory);
                        this.accessories[uuid] = dingzDaAccessory;
                        return dingzDaAccessory;
                    }
                    catch (e) {
                        return Promise.reject();
                    }
                }
                else {
                    this.log.warn('Accessory', chalk_1.default.magentaBright(`[${deviceInfo.name}]`), `(${address}) already initialized`);
                    // Update Names et al. from new device info
                    // Might be needed e.g. when IP address changes
                    this.eb.emit("updateDeviceInfo" /* PlatformEvent.UPDATE_DEVICE_INFO */, deviceInfo);
                    this.accessories[uuid].identify();
                    return this.accessories[uuid];
                }
            }
            else {
                return Promise.reject();
            }
        })
            .catch((e) => {
            if (e instanceof errors_1.DeviceNotReachableError) {
                this.log.error(`addDingzDevice() --> Device not reachable -> ${name} (${address})`);
            }
            if (existingAccessory) {
                // Here we have a problem: The accessory has been loaded but not initialized
                const dingzDaAccessory = new dingzAccessory_1.DingzAccessory(this, existingAccessory);
                this.accessories[existingAccessory.UUID] = dingzDaAccessory;
                return dingzDaAccessory;
            }
            return Promise.reject(e);
        });
    }
    // Add one device based on address and name
    addMyStromSwitchDevice({ address, name = 'Switch', token, }) {
        // Run a diacovery of changed things every 10 seconds
        this.log.debug(`Add configured/discovered device -> ${name} (${address})`);
        const success = this.getMyStromDeviceInfo({
            address,
            token,
            endpoint: 'info', // We use the old endpoint
        }).then((data) => {
            var _a;
            if (typeof data !== 'undefined') {
                const info = data;
                if (
                // FIXME: Fix the API Documentation
                // The type info returned varies with Firmware versions.
                // Newer Firmwares seem to have string-based types whereas
                // older ones use the numbers from the API documentation
                info.type !== 'WS2' &&
                    info.type !== commonTypes_1.DeviceTypes.MYSTROM_SWITCH_CHV2 &&
                    info.type !== 'WSEU' &&
                    info.type !== commonTypes_1.DeviceTypes.MYSTROM_SWITCH_EU &&
                    info.type !== 'LCS' &&
                    info.type !== undefined // Switch V1 does not have a type
                ) {
                    throw new errors_1.InvalidTypeError(`Device ${name} at ${address} is of the wrong type (${info.type} instead of "myStrom Switch")`);
                }
                const deviceInfo = {
                    name: name,
                    address: address,
                    mac: info.mac.toUpperCase(),
                    token: token,
                    model: (_a = myStromTypes_1.MyStromSwitchTypes[info.type]) !== null && _a !== void 0 ? _a : 'CH v1',
                    hwInfo: info,
                    accessoryClass: 'MyStromSwitchAccessory',
                };
                const uuid = this.api.hap.uuid.generate(deviceInfo.mac);
                // check that the device has not already been registered by checking the
                // cached devices we stored in the `configureAccessory` method above
                if (!this.accessories[uuid]) {
                    this.log.info('Registering new accessory:', deviceInfo.name);
                    // create a new accessory
                    const accessory = new this.api.platformAccessory(deviceInfo.name, uuid);
                    // store a copy of the device object in the `accessory.context`
                    // the `context` property can be used to store any data about the accessory you may need
                    accessory.context.device = deviceInfo;
                    // create the accessory handler (which will add services as needed)
                    // this is imported from `dingzDaAccessory.ts`
                    const myStromSwitchAccessory = new myStromSwitchAccessory_1.MyStromSwitchAccessory(this, accessory);
                    // link the accessory to your platform
                    this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                        accessory,
                    ]);
                    // push into accessory cache
                    this.accessories[uuid] = myStromSwitchAccessory;
                    return true;
                }
                else {
                    this.log.warn('Accessory', chalk_1.default.magentaBright(`[${deviceInfo.name}]`), `(${address}) already initialized`);
                    this.eb.emit("updateDeviceInfo" /* PlatformEvent.UPDATE_DEVICE_INFO */, deviceInfo);
                    this.accessories[uuid].identify();
                    return true;
                }
            }
        });
        // Nothing found, throw error
        if (!success) {
            throw new errors_1.DeviceNotReachableError(`Device not found -> ${name} (${address})`);
        }
        return true;
    }
    // Add one device based on address and name
    addMyStromLightbulbDevice({ address, name = 'Lightbulb/LED', token, }) {
        // Run a diacovery of changed things every 10 seconds
        this.log.debug(`Add configured/discovered device -> ${name} (${address})`);
        const success = this.getMyStromDeviceInfo({
            address,
            token,
        }).then((data) => {
            var _a;
            if (typeof data !== 'undefined') {
                const info = data;
                if (info.type !== commonTypes_1.DeviceTypes.MYSTROM_BULB &&
                    info.type !== commonTypes_1.DeviceTypes.MYSTROM_LEDSTRIP &&
                    info.type !== 'WRS') {
                    throw new errors_1.InvalidTypeError(`Device ${name} at ${address} is of the wrong type (${info.type} instead of "myStrom Lightbulb")`);
                }
                const deviceInfo = {
                    name: (_a = info.name) !== null && _a !== void 0 ? _a : name,
                    address: address,
                    mac: info.mac.toUpperCase(),
                    token: token,
                    model: commonTypes_1.DeviceTypes.MYSTROM_BULB.toString(),
                    hwInfo: info,
                    accessoryClass: 'MyStromLightbulbAccessory',
                };
                const uuid = this.api.hap.uuid.generate(deviceInfo.mac);
                // check that the device has not already been registered by checking the
                // cached devices we stored in the `configureAccessory` method above
                if (!this.accessories[uuid]) {
                    this.log.info('Registering new accessory:', deviceInfo.name);
                    // create a new accessory
                    const accessory = new this.api.platformAccessory(deviceInfo.name, uuid);
                    // store a copy of the device object in the `accessory.context`
                    // the `context` property can be used to store any data about the accessory you may need
                    accessory.context.device = deviceInfo;
                    // create the accessory handler (which will add services as needed)
                    // this is imported from `dingzDaAccessory.ts`
                    const myStromLightbulbAccessory = new myStromLightbulbAccessory_1.MyStromLightbulbAccessory(this, accessory);
                    // link the accessory to your platform
                    this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                        accessory,
                    ]);
                    // push into accessory cache
                    this.accessories[uuid] = myStromLightbulbAccessory;
                    return true;
                }
                else {
                    this.log.warn('Accessory', chalk_1.default.magentaBright(`[${deviceInfo.name}]`), `(${address}) already initialized`);
                    this.eb.emit("updateDeviceInfo" /* PlatformEvent.UPDATE_DEVICE_INFO */, deviceInfo);
                    this.accessories[uuid].identify();
                    return true;
                }
            }
        });
        // Nothing found, throw error
        if (!success) {
            throw new errors_1.DeviceNotReachableError(`Device not found -> ${name} (${address})`);
        }
        return true;
    }
    // Add one device based on address and name
    addMyStromPIRDevice({ address, name, token, }) {
        // Run a diacovery of changed things every 10 seconds
        this.log.debug(`Add configured/discovered myStrom PIR device -> (${address})`);
        const success = this.getMyStromDeviceInfo({
            address,
            token,
        }).then((data) => {
            if (typeof data !== 'undefined') {
                const info = data;
                // Need this to identify the right type
                if (info.type !== commonTypes_1.DeviceTypes.MYSTROM_PIR) {
                    throw new errors_1.InvalidTypeError(`Device ${name} at ${address} is of the wrong type (${info.type} instead of "myStrom Lightbulb")`);
                }
                const pirName = name !== null && name !== void 0 ? name : 'my PIR ' + info.mac.toUpperCase().substr(6, 6);
                const deviceInfo = {
                    name: pirName,
                    address: address,
                    mac: info.mac.toUpperCase(),
                    token: token,
                    model: commonTypes_1.DeviceTypes.MYSTROM_PIR.toString(),
                    hwInfo: info,
                    accessoryClass: 'MyStromPIRAccessory',
                };
                const uuid = this.api.hap.uuid.generate(deviceInfo.mac);
                // check that the device has not already been registered by checking the
                // cached devices we stored in the `configureAccessory` method above
                if (!this.accessories[uuid]) {
                    this.log.info('Registering new accessory:', deviceInfo.name);
                    // create a new accessory
                    const accessory = new this.api.platformAccessory(deviceInfo.name, uuid);
                    // store a copy of the device object in the `accessory.context`
                    // the `context` property can be used to store any data about the accessory you may need
                    accessory.context.device = deviceInfo;
                    // create the accessory handler (which will add services as needed)
                    // this is imported from `dingzDaAccessory.ts`
                    const myStromPIRAccessory = new myStromPIRAccessory_1.MyStromPIRAccessory(this, accessory);
                    // link the accessory to your platform
                    this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                        accessory,
                    ]);
                    // push into accessory cache
                    this.accessories[uuid] = myStromPIRAccessory;
                    return true;
                }
                else {
                    this.log.warn('Accessory', chalk_1.default.magentaBright(`[${deviceInfo.name}]`), `(${address}) already initialized`);
                    this.eb.emit("updateDeviceInfo" /* PlatformEvent.UPDATE_DEVICE_INFO */, deviceInfo);
                    this.accessories[uuid].identify();
                    return true;
                }
            }
        });
        // Nothing found, throw error
        if (!success) {
            throw new errors_1.DeviceNotReachableError(`Device not found -> ${name} (${address})`);
        }
        return true;
    }
    // Add one device based on address and name
    addMyStromButtonDevice({ address, name = 'Button', token, mac, }) {
        // Run a discovery of changed things every 10 seconds
        this.log.debug(`Add configured/discovered myStrom Button device -> ${name} (${address})`);
        const deviceInfo = {
            name: name,
            address: address,
            mac: mac === null || mac === void 0 ? void 0 : mac.toUpperCase(),
            token: token,
            model: commonTypes_1.DeviceTypes.MYSTROM_BUTTON.toString(),
            accessoryClass: 'MyStromButtonAccessory',
        };
        const uuid = this.api.hap.uuid.generate(deviceInfo.mac);
        // check that the device has not already been registered by checking the
        // cached devices we stored in the `configureAccessory` method above
        if (!this.accessories[uuid]) {
            this.log.info('Registering new accessory:', name);
            // create a new accessory
            const accessory = new this.api.platformAccessory(name, uuid);
            // store a copy of the device object in the `accessory.context`
            // the `context` property can be used to store any data about the accessory you may need
            accessory.context.device = deviceInfo;
            // create the accessory handler (which will add services as needed)
            // this is imported from `dingzDaAccessory.ts`
            const myStromButtonAccessory = new myStromButtonAccessory_1.MyStromButtonAccessory(this, accessory);
            // link the accessory to your platform
            this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                accessory,
            ]);
            // push into accessory cache
            this.accessories[uuid] = myStromButtonAccessory;
            // Buttons can be initialized without fetching all Info -- however it would be good to fetch it anyway
            this.getMyStromDeviceInfo({
                address,
                token,
            }).then((data) => {
                if (typeof data !== 'undefined') {
                    const info = data;
                    if (info.type !== commonTypes_1.DeviceTypes.MYSTROM_BUTTON) {
                        throw new errors_1.InvalidTypeError(`Device ${name} at ${address} is of the wrong type (${info.type} instead of "myStrom Button")`);
                    }
                    accessory.context.device.hwInfo = info;
                }
            });
            return true;
        }
        else {
            this.log.warn('Accessory', chalk_1.default.magentaBright(`[${deviceInfo.name}]`), `(${address}) already initialized`);
            this.eb.emit("updateDeviceInfo" /* PlatformEvent.UPDATE_DEVICE_INFO */, deviceInfo);
            this.accessories[uuid].identify();
            return true;
        }
    }
    // Add one device based on address and name
    addMyStromButtonPlusDevice({ address, name = 'Button+', token, mac, }) {
        // Run a discovery of changed things every 10 seconds
        this.log.debug(`Add configured/discovered myStrom Button+ (2nd Gen) device -> ${name} (${address})`);
        const deviceInfo = {
            name: name,
            address: address,
            mac: mac === null || mac === void 0 ? void 0 : mac.toUpperCase(),
            token: token,
            model: commonTypes_1.DeviceTypes.MYSTROM_BUTTON_PLUS_2G.toString(),
            accessoryClass: 'MyStromButtonPlusAccessory',
        };
        const uuid = this.api.hap.uuid.generate(deviceInfo.mac);
        // check that the device has not already been registered by checking the
        // cached devices we stored in the `configureAccessory` method above
        if (!this.accessories[uuid]) {
            this.log.info('Registering new accessory:', name);
            // create a new accessory
            const accessory = new this.api.platformAccessory(name, uuid);
            // store a copy of the device object in the `accessory.context`
            // the `context` property can be used to store any data about the accessory you may need
            accessory.context.device = deviceInfo;
            // create the accessory handler (which will add services as needed)
            // this is imported from `dingzDaAccessory.ts`
            const myStromButtonPlusAccessory = new myStromButtonPlusAccessory_1.MyStromButtonPlusAccessory(this, accessory);
            // link the accessory to your platform
            this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                accessory,
            ]);
            // push into accessory cache
            this.accessories[uuid] = myStromButtonPlusAccessory;
            // Buttons can be initialized without fetching all Info -- however it would be good to fetch it anyway
            this.getMyStromDeviceInfo({
                address,
                token,
            }).then((data) => {
                if (typeof data !== 'undefined') {
                    const info = data;
                    if (info.type !== commonTypes_1.DeviceTypes.MYSTROM_BUTTON_PLUS_2G) {
                        throw new errors_1.InvalidTypeError(`Device ${name} at ${address} is of the wrong type (${info.type} instead of "myStrom Button+ (2nd Gen)")`);
                    }
                    accessory.context.device.hwInfo = info;
                }
            });
            return true;
        }
        else {
            this.log.warn('Accessory', chalk_1.default.magentaBright(`[${deviceInfo.name}]`), `(${address}) already initialized`);
            this.eb.emit("updateDeviceInfo" /* PlatformEvent.UPDATE_DEVICE_INFO */, deviceInfo);
            this.accessories[uuid].identify();
            return true;
        }
    }
    /**
     * Parses and interprets auto-discovery messages
     * @param msg
     * @param remoteInfo
     */
    autoDiscoveryMessageHandler(msg, remoteInfo) {
        // const mac: string = dataBuffer.toString('hex', 0, 6);
        try {
            if (msg.length !== 8) {
                throw new errors_1.DeviceNotImplementedError('Detected data can not be parsed.');
            }
            const t = msg[6];
            const mac = this.byteToHexString(msg.subarray(0, 6));
            const deviceSuffix = mac.substr(6, 6);
            // If auto-discovery is disabled, we will return, however
            // the MAC of the discovered device will be printed in debug
            if (!this.config.autoDiscover) {
                this.log.debug(`Auto-discovery disabled: ignoring discovered device ${mac} at ${remoteInfo.address}`);
                return;
            }
            // Check if already discovered, and if not ignored
            // Implements #497
            if (this.ignored.has(mac.toUpperCase())) {
                if (!this.ignored.get(mac.toUpperCase()).isignored) {
                    this.log.info('Ignoring discovered device', this.ignored.get(mac.toUpperCase()).comment || '', 'at', mac.toUpperCase());
                    this.ignored.get(mac.toUpperCase()).isignored = true;
                }
            }
            else {
                if (!this.discovered.has(mac)) {
                    switch (t) {
                        case commonTypes_1.DeviceTypes.MYSTROM_BUTTON_PLUS:
                            throw new errors_1.DeviceNotImplementedError(`Device discovered at ${remoteInfo.address} (MAC:${mac.toUpperCase()}) of unsupported type ${commonTypes_1.DeviceTypes[t]} `);
                            break;
                        case commonTypes_1.DeviceTypes.MYSTROM_BUTTON:
                            retryWithBreaker
                                .execute(() => {
                                this.addMyStromButtonDevice({
                                    address: remoteInfo.address,
                                    name: `Button ${deviceSuffix}`,
                                    token: this.config.globalToken,
                                    mac: mac,
                                });
                            })
                                .then(() => {
                                this.discovered.set(mac, remoteInfo);
                            });
                            break;
                        case commonTypes_1.DeviceTypes.MYSTROM_BUTTON_PLUS_2G:
                            retryWithBreaker
                                .execute(() => {
                                this.addMyStromButtonPlusDevice({
                                    address: remoteInfo.address,
                                    name: `Button+ ${deviceSuffix}`,
                                    token: this.config.globalToken,
                                    mac: mac,
                                });
                            })
                                .then(() => {
                                this.discovered.set(mac, remoteInfo);
                            });
                            break;
                        case commonTypes_1.DeviceTypes.MYSTROM_LEDSTRIP:
                            retryWithBreaker
                                .execute(() => {
                                this.addMyStromLightbulbDevice({
                                    address: remoteInfo.address,
                                    name: `LED Strip ${deviceSuffix}`,
                                    token: this.config.globalToken,
                                });
                            })
                                .then(() => {
                                this.discovered.set(mac, remoteInfo);
                            });
                            break;
                        case commonTypes_1.DeviceTypes.MYSTROM_BULB:
                            retryWithBreaker
                                .execute(() => {
                                this.addMyStromLightbulbDevice({
                                    address: remoteInfo.address,
                                    name: `Lightbulb ${deviceSuffix}`,
                                    token: this.config.globalToken,
                                });
                            })
                                .then(() => {
                                this.discovered.set(mac, remoteInfo);
                            });
                            break;
                        case commonTypes_1.DeviceTypes.MYSTROM_SWITCH_CHV1:
                        case commonTypes_1.DeviceTypes.MYSTROM_SWITCH_CHV2:
                        case commonTypes_1.DeviceTypes.MYSTROM_SWITCH_EU:
                        case commonTypes_1.DeviceTypes.MYSTROM_SWITCH_ZERO:
                            retryWithBreaker
                                .execute(() => {
                                this.addMyStromSwitchDevice({
                                    address: remoteInfo.address,
                                    name: `Switch ${deviceSuffix}`,
                                    token: this.config.globalToken,
                                });
                            })
                                .then(() => {
                                this.discovered.set(mac, remoteInfo);
                            });
                            break;
                        case commonTypes_1.DeviceTypes.MYSTROM_PIR:
                            retryWithBreaker
                                .execute(() => {
                                this.addMyStromPIRDevice({
                                    address: remoteInfo.address,
                                    name: `PIR ${deviceSuffix}`,
                                    token: this.config.globalToken,
                                });
                            })
                                .then(() => {
                                this.discovered.set(mac, remoteInfo);
                            });
                            break;
                        case commonTypes_1.DeviceTypes.DINGZ:
                            retryWithBreaker
                                .execute(() => {
                                this.addDingzDevice({
                                    address: remoteInfo.address,
                                    name: `DINGZ ${deviceSuffix}`,
                                    token: this.config.globalToken,
                                });
                            })
                                .then(() => {
                                this.discovered.set(mac, remoteInfo);
                            })
                                .catch((e) => this.handleError.bind(this, e));
                            break;
                        default:
                            this.log.warn(`Unknown device: ${t}`);
                            break;
                    }
                }
                else {
                    this.log.debug(`Stopping discovery of already known device: ${mac.toUpperCase()}`);
                }
            }
        }
        catch (e) {
            if (e instanceof errors_1.DeviceNotImplementedError) {
                // Degrade gracefully if type not found
                this.log.warn(e.message);
            }
            else if (e instanceof errors_1.DeviceNotReachableError) {
                this.log.warn(e.message);
            }
            else {
                throw e;
            }
        }
    }
    // Steup device discovery. This will run for 10 minutes and then stop
    // If you want tore-discover, just restart Homebridge
    setupDeviceDiscovery() {
        const discoverySocket = (0, dgram_1.createSocket)({
            type: 'udp4',
        });
        this.log.info('Starting discovery');
        try {
            process.on('exit', () => {
                try {
                    discoverySocket.close();
                }
                catch (e) {
                    if (e.code === 'ERR_SOCKET_DGRAM_NOT_CONNECTED') {
                        this.log.info('Socket already destroyed');
                    }
                }
                finally {
                    this.log.info('Process ended, socket destroyed');
                }
            });
            discoverySocket
                .on('message', this.autoDiscoveryMessageHandler.bind(this))
                .bind(settings_1.DINGZ_DISCOVERY_PORT);
            setTimeout(() => {
                this.log.info('Stopping discovery');
                try {
                    discoverySocket.close();
                }
                catch (e) {
                    if (e.code === 'ERR_SOCKET_DGRAM_NOT_CONNECTED') {
                        this.log.info('Socket already destroyed');
                    }
                }
            }, 600000); // Discover for 10 min then stop
            // Make sure we close the socket even if we get killed
        }
        catch (e) {
            this.log.error(e.code + ' Socket error');
        }
        return true;
    }
    getAccessoryByMac(mac) {
        const uuid = this.api.hap.uuid.generate(mac.toUpperCase());
        return this.accessories[uuid];
    }
    // Set the callback URL for the device
    // FIXME: Move this to a platform accessory base class
    async setButtonCallbackUrl({ baseUrl, token, oldUrl, endpoints, }) {
        const setActionUrl = `${baseUrl}/api/v1/action/`;
        let callbackUrl = this.getCallbackUrl();
        if (oldUrl === null || oldUrl === void 0 ? void 0 : oldUrl.endsWith('||')) {
            callbackUrl = `${oldUrl}${callbackUrl}`;
        }
        else if (oldUrl) {
            callbackUrl = `${oldUrl}||${callbackUrl}`;
        }
        this.log.debug('Setting the callback URL -> ', callbackUrl);
        endpoints.forEach((endpoint) => {
            this.log.debug(setActionUrl, 'Endpoint -> ', endpoint);
            axios_1.default
                .post(`${setActionUrl}${endpoint}`, callbackUrl, {
                headers: { Token: token !== null && token !== void 0 ? token : '' },
            })
                .catch(this.handleError.bind(this));
        });
    }
    // Create a Service to listen for dingz Button events
    callbackServer() {
        var _a;
        this.app.use(bodyParser.urlencoded());
        this.app.post('/button', this.handleRequest.bind(this));
        this.app.listen((_a = this.config.callbackPort) !== null && _a !== void 0 ? _a : settings_1.DINGZ_CALLBACK_PORT, () => {
            var _a;
            return this.log.warn(`Callback server listening for POST requests on ${(_a = this.config.callbackPort) !== null && _a !== void 0 ? _a : settings_1.DINGZ_CALLBACK_PORT}... use ${this.getCallbackUrl()} as URL for your dingz or MyStrom callbacks`);
        });
    }
    handleRequest(request, response) {
        if (request.url) {
            response.writeHead(204); // 204 No content
            response.end(() => {
                var _a, _b;
                this.log.debug(chalk_1.default.blueBright('[ACTION]'), 'Incoming request ->', request.body);
                const b = request.body;
                const mac = (_a = b.mac) !== null && _a !== void 0 ? _a : '';
                const module = b.index;
                const action = b.action;
                const battery = (_b = b.battery) !== null && _b !== void 0 ? _b : b.bat; // Difference between myStrom Buttons and Buttons+ (2nd Gen)
                const temperature = b.temp;
                const humidity = b.rh;
                // Various types of callbacks
                /**
                 * module, battery, humidity & temperature: myStrom Button+ (2nd Gen)
                 * module: Dingz
                 * action: myStrom Button
                 * everything else: heartbeat
                 */
                if (module && battery && temperature && humidity) {
                    // MyStrom Button+
                    this.log.info(chalk_1.default.blueBright('[ACTION]'), `Module ${request.ip} (${module}/${action})`);
                    // attempt auto-add of a dingz
                    if (request.socket.remoteAddress &&
                        (0, is_valid_host_1.default)(request.socket.remoteAddress)) {
                        this.addMyStromButtonPlusDevice({
                            address: request.socket.remoteAddress,
                            name: `BUTTON+ ${mac.substring(6, 12)}`,
                            mac: mac,
                        });
                    }
                    this.eb.emit("deviceAction" /* PlatformEvent.ACTION */, mac, action, module, battery, temperature, humidity);
                }
                else if (module) {
                    // Dingz
                    this.log.info(chalk_1.default.blueBright('[ACTION]'), `Module ${request.ip} (${module}/${action})`);
                    // attempt auto-add of a dingz
                    if (request.socket.remoteAddress &&
                        (0, is_valid_host_1.default)(request.socket.remoteAddress)) {
                        this.addDingzDevice({
                            address: request.socket.remoteAddress,
                            name: `DINGZ ${mac.substring(6, 12)}`,
                        });
                    }
                    this.eb.emit("deviceAction" /* PlatformEvent.ACTION */, mac, action, module);
                }
                else if (action) {
                    // myStrom Button
                    this.log.info(chalk_1.default.blueBright('[ACTION] Simple'), `Button ${request.ip} (${action})`);
                    this.eb.emit("deviceAction" /* PlatformEvent.ACTION */, mac, action, battery);
                }
                else {
                    // Everything else (i.e. heartbeat)
                    this.log.info(chalk_1.default.blueBright('[HEARTBEAT]'), request.ip);
                    this.eb.emit("deviceAction" /* PlatformEvent.ACTION */, mac, action, battery);
                }
            });
        }
    }
    getCallbackUrl() {
        var _a, _b;
        const hostname = (_a = this.config.callbackHostname) !== null && _a !== void 0 ? _a : os.hostname();
        const port = (_b = this.config.callbackPort) !== null && _b !== void 0 ? _b : settings_1.DINGZ_CALLBACK_PORT;
        return `post://${hostname}:${port}/button`;
    }
    /**
     * Device Methods -- these are used to retrieve the data from the dingz
     * FIXME: API Endpoint
     * Officially, the API is at /api/v1/info but there's
     * an undocumenten API at /info which also works for V1 switches
     */
    async getMyStromDeviceInfo({ address, token, endpoint = 'api/v1/info', }) {
        this.log.debug('Fetching myStrom Device Info:', address);
        const deviceInfoUrl = `http://${address}/${endpoint}`;
        return await (0, axios_1.default)(deviceInfoUrl, {
            headers: {
                Token: token !== null && token !== void 0 ? token : '',
            },
        })
            .then((response) => {
            return response.data;
        })
            .catch(this.handleError.bind(this));
    }
    byteToHexString(uint8arr) {
        if (!uint8arr) {
            return '';
        }
        let hexStr = '';
        for (let i = 0; i < uint8arr.length; i++) {
            let hex = (uint8arr[i] & 0xff).toString(16);
            hex = hex.length === 1 ? '0' + hex : hex;
            hexStr += hex;
        }
        return hexStr.toUpperCase();
    }
}
exports.DingzDaHomebridgePlatform = DingzDaHomebridgePlatform;
//# sourceMappingURL=platform.js.map